import os
import sqlite3

DATABASE_NAME = 'contacts.db'

TOPLEVEL_FIELD = 'o'
MIDDLELEVEL_FIELD = 'ou'
LOWLEVEL_FIELD = 'cn'

SCHEMA = """
    begin;

    create table data(
        id integer,
        field_id text,
        value text
    );

    create table fields(
        id integer primary key,
        name text
    );

    create table struct(
        id integer primary key,
        name text,
        parent integer
    );

    commit;

    create index i_fields on fields (id);
    create index i_data on data (id);
    create index i_struct on struct(parent);

    commit;

"""


class SQLite:
    def __init__(self, basedir):
        self._path = os.path.join(basedir, DATABASE_NAME)
        self.conn = None
        if not os.path.exists(self._path):
            self.new()
        else:
            self.conn = sqlite3.connect(self._path)

    def new(self):
        """Creates new databse."""

        self.conn = sqlite3.connect(self._path)
        self.conn.executescript(SCHEMA)
        self.conn.commit()

    def close(self):
        """Closes connection with database."""

        self.conn.commit()
        self.conn.close()

    def save(self):
        """Save all changes."""

        self.conn.commit()

    def clear(self):
        """Clear all database tables."""

        execute = self.conn.execute
        execute("DELETE from data")
        execute("DELETE from fields")
        execute("DELETE from struct")
        self.conn.commit()

    def get_fields(self):
        """Returns all fields from FIELDS table."""

        return [item[0] for item in self.conn.execute( \
            "SELECT name FROM fields").fetchall()]

    # operations with DATA table
    def add_entry(self, entry):
        """Adds new entry to database."""

        execute = self.conn.execute

        if not 'uid' in entry or not 'o' in entry:
            return

        try:
            _id = execute("SELECT MAX(id) FROM data").fetchone()[0] \
                + 1
        except TypeError:
            _id = 1

        for field, values in entry.items():
            # update FIELDS table
            if execute("SELECT name FROM fields WHERE name=?", \
                (field,)).fetchone() is None:
                execute("INSERT INTO fields values(NULL, ?)", (field,))

            # update DATA table
            for value in values:
                execute("INSERT INTO data values(?,?,?)", (_id, field, value))

        # update STRUCT table
        name = entry[TOPLEVEL_FIELD][0]
        row_id = execute("SELECT id FROM struct WHERE name=? ", \
            (name,)).fetchone()
        if row_id is None: # update STRUCT table (TOPLEVEL_FIELD)
            execute("INSERT INTO struct values(NULL,?,0)", (name,))
            row_id = execute("SELECT last_insert_rowid()").fetchone()[0]
        else:
            row_id = row_id[0]

        name = entry[MIDDLELEVEL_FIELD][0]
        sql_res = execute("SELECT id FROM struct WHERE name=? AND parent=?", \
            (name, row_id)).fetchone()
        if sql_res is None: # update STRUCT table (MIDDLELEVEL_FIELD)
             execute("INSERT INTO struct values(NULL,?,?)", (name, row_id))

    def get_entry(self, value):
        """Gets full entry imformation from DATA table."""

        _id = self.conn.execute("SELECT id FROM data WHERE value=?", \
            (value,)).fetchone()[0]
        return dict([(key, value) for key, value in self.conn.execute("\
            SELECT field_id, value FROM data WHERE id=?", (_id,)).fetchall()])

    def get_toplevel_entries(self):
        """Returns list of all toplevel entries from DATA table."""

        return [item[0] for item in self.conn.execute("SELECT name \
            FROM struct WHERE parent=0 ORDER BY name ASC").fetchall()]

    def get_middlelevel_entries(self, parent=None):
        """Returns list of all middlelevel entries from DATA table."""

        if not parent:
            return [item[0] for item in self.conn.execute("SELECT DISTINCT \
                name FROM struct WHERE parent!=0 ORDER BY name ASC").fetchall()]
        else:
            return [item[0] for item in self.conn.execute("SELECT name FROM \
                struct WHERE parent IN (SELECT id FROM struct WHERE name=?) \
                ORDER BY name ASC", (parent,)).fetchall()]

    def get_lowlevel_entries(self, parent=None):
        """Returns list of all lowlevel entries from DATA table."""

        if not parent:
            return [item[0] for item in self.conn.execute("SELECT DISTINCT \
                value FROM data WHERE field_id=? ORDER BY value ASC", \
                (LOWLEVEL_FIELD,)).fetchall()]
        else:
            print parent
            return [item[0] for item in self.conn.execute("SELECT value FROM \
                data WHERE id IN (SELECT id FROM data WHERE value=?) AND field_id='cn' \
                ORDER BY value ASC", (parent, )).fetchall()]

    def get_entry_by_id(self, _id):
        """Returns full entry by it id."""

        sql_res = self.conn.execute("""SELECT id_field, value FROM data 
            WHERE id=?""", (_id,)).fetchall()
        return dict([(self.get_field_name_by_id(field_id), value) for field_id, \
            value in sql_res])

    def get_all_entries(self):
        """Returns list of all entries."""

        ids = [item[0] for item in self.conn.execute("""SELECT DISTINCT id \
            FROM data""").fetchall()]
        return [self.get_entry_by_id(_id) for _id in ids]




if __name__ == "__main__":
    db = SQLite('/tmp/')
    #db.update_field('test')
    #print db.get_field_id_by_name('fax')
    #print db.get_entry_by_id(1)
    #print db.get_all_entries()
    db.close()


