"""
Hildon UI for Meabook
"""

import gtk
import hildon
import gettext
from meabook.config import Config
from meabook.database.SQLite import SQLite

_ = gettext.gettext



def create_button(title, value):
    """Create HildonButton."""

    button = hildon.Button(gtk.HILDON_SIZE_AUTO | \
        gtk.HILDON_SIZE_FINGER_HEIGHT, \
        hildon.BUTTON_ARRANGEMENT_VERTICAL, title, value)
    button.set_style(hildon.BUTTON_STYLE_PICKER)
    button.set_alignment(0, 0, 0, 0)
    return button


class HildonMeabook():
    def __init__(self, basedir):
        self.database = SQLite(basedir)
        self.config = Config(basedir, self.database)
        self.window = hildon.StackableWindow()
        self.window.set_title(_('Meabook'))
        self.window.connect('destroy', self.exit)

        # create menu buttons
        menu = hildon.AppMenu()
        settings_button = hildon.Button(gtk.HILDON_SIZE_AUTO, \
            hildon.BUTTON_ARRANGEMENT_HORIZONTAL, _('Settings'))
        about_button = hildon.Button(gtk.HILDON_SIZE_AUTO, \
            hildon.BUTTON_ARRANGEMENT_HORIZONTAL, _('About'))

        # create filter widgets
        self.level1_filter = gtk.RadioButton(None, _('Filiation'))
        self.level2_filter = gtk.RadioButton(self.level1_filter, _('Depart'))
        self.level3_filter = gtk.RadioButton(self.level1_filter, _('People'))
        for filter_widget in [self.level1_filter, self.level2_filter, \
            self.level3_filter]:
            filter_widget.set_mode(False)

        # create items list
        self.selector = hildon.TouchSelector(text=True)

        # connect signals
        self.callback_id = self.selector.connect('changed', \
            self.show_middlelevel_dialog)
        settings_button.connect('clicked', self.show_settings_dialog)
        about_button.connect('clicked', self.show_about_dialog)
        self.level1_filter.connect('clicked', self.apply_filter, \
            self.database.get_toplevel_entries)
        self.level2_filter.connect('clicked', self.apply_filter, \
            self.database.get_middlelevel_entries)
        self.level3_filter.connect('clicked', self.apply_filter, \
            self.database.get_lowlevel_entries)

        # fill items list
        for filter_widget in [self.level1_filter, self.level2_filter, \
            self.level3_filter]:
            if filter_widget.get_active():
                self.apply_filter(filter_widget, self.database.get_toplevel_entries)
                break

        # packing widgets
        self.window.add(self.selector)
        menu.add_filter(self.level1_filter)
        menu.add_filter(self.level2_filter)
        menu.add_filter(self.level3_filter)
        menu.append(settings_button)
        menu.append(about_button)
        menu.show_all()
        self.window.set_app_menu(menu)
        self.window.show_all()

    def start(self):
        """Start the program."""

        gtk.main()

    def exit(self, event):
        """Exit the program."""

        self.database.close()
        gtk.main_quit()

    def show_settings_dialog(self, widget):
        """Shows Config dialog."""

        dialog = ConfigurationDialog(self.database, self.config)
        dialog.run()

    def show_about_dialog(self, widget):
        """Shows About dialog."""

        dialog = hildon.Dialog()
        dialog.set_title(_('About'))
        label = gtk.Label('\nMeabook v0.1\n')
        dialog.vbox.add(label)
        dialog.vbox.show_all()
        dialog.run()
        dialog.destroy()

    def apply_filter(self, widget, db_function):
        """Updates toplevel selector with different level items."""

        # blocks 'change' event for selector
        self.selector.handler_block(self.callback_id)

        # update list content
        self.selector.get_model(0).clear()
        for item in db_function():
            self.selector.append_text(item)
        self.selector.unselect_all(0)

        self.window.set_title(' - '.join([_('Meabook'), widget.get_label()]))
        # reconnect signal. depends on current filter
        if widget == self.level1_filter:
            self.callback_id = self.selector.connect('changed', \
                self.show_middlelevel_dialog)
        elif widget == self.level2_filter:
            self.callback_id = self.selector.connect('changed', \
                self.show_lowlevel_dialog)
        else:
            self.callback_id = self.selector.connect('changed', \
                self.show_item_dialog)

    def _create_items_dialog(self, title):
        """Creates dialog for showing items."""

        selector = hildon.TouchSelector(text=True)
        window = hildon.StackableWindow()
        window.set_title(title)
        window.add(selector)
        return window, selector

    def show_middlelevel_dialog(self, widget, column):
        """Shows subitems of selected level1-item."""

        selected_item = widget.get_current_text()
        window, selector = self._create_items_dialog(selected_item)
        selector.connect('changed', self.show_lowlevel_dialog)
        for item in self.database.get_middlelevel_entries(selected_item):
            selector.append_text(item)
        window.show_all()

    def show_lowlevel_dialog(self, widget, column):
        """Shows lowlevel items."""

        selected_item = widget.get_current_text()
        window, selector = self._create_items_dialog(selected_item)
        selector.connect('changed', self.show_item_dialog)
        for item in self.database.get_lowlevel_entries():
            selector.append_text(item)
        window.show_all()

    def show_item_dialog(self, widget, column):
        """Shows detailed item information."""

        selected_item = widget.get_current_text()
        entry = self.database.get_entry(selected_item)
        window = hildon.StackableWindow()
        window.set_title(selected_item)
        pannable_area = hildon.PannableArea()
        vbox = gtk.VBox()

        for key, value in entry.items():
            button = create_button(_(key) , value)
            vbox.pack_start(button)

        pannable_area.add_with_viewport(vbox)
        window.add(pannable_area)
        window.show_all()




class ConfigurationDialog:
    """Configuration dialog"""

    def __init__(self, database, config):
        self.config = config
        self.database = database
        self.window = hildon.StackableWindow()
        self.window.set_title(_('Configuration'))

        # create menu
        menu = hildon.AppMenu()
        self.button_order = gtk.RadioButton(None, _('Order'))
        self.button_order.set_mode(False)
        self.button_order.connect('clicked', self.show_order_settings, \
            self.database.get_fields())
        self.button_items = gtk.RadioButton(self.button_order, _('Items'))
        self.button_items.set_mode(False)
        self.button_items.connect('clicked', self.show_items_settings, \
            self.database.get_fields())
        menu.add_filter(self.button_items)
        menu.add_filter(self.button_order)
        menu.show_all()

        # create other widgets
        self.window.set_app_menu(menu)
        self.selector = hildon.TouchSelector(text=True)
        self.callback_id = self.selector.connect('changed', \
            self.save_selected_fields)
        self.pannable_area = hildon.PannableArea()
        vbox = gtk.VBox()
        vbox.pack_start(self.selector)
        vbox.pack_start(self.pannable_area)
        self.window.add(vbox)

    def show_items_settings(self, widget, items):
        """Shows dialog for selecting necessary items."""

        self.pannable_area.hide()
        # blocks 'change' event for selector
        self.selector.handler_block(self.callback_id)

        # fill items list
        for item in items:
            self.selector.append_text(_(item))

        self.selector.set_column_selection_mode( \
                hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
        self.selector.unselect_all(0)

        # mark necessary items
        items_to_select = self.config.get_fields()
        model = self.selector.get_model(0)
        for index, item in enumerate(items):
            if item in items_to_select:
                self.selector.select_iter(0, model.get_iter(index), False)

        self.selector.show()

        # reconnect signal
        self.selector.connect('changed', self.save_selected_fields)

    def show_order_settings(self, widget, items):
        """Shows dialog for selecting items order."""

        self.selector.hide()
        vbox = gtk.VBox()
        for item in items:
            button = create_button(item, '')
            button.set_style(hildon.BUTTON_STYLE_NORMAL)
            button.set_alignment(0, 0.5, 0, 0)
            vbox.pack_start(button)
        self.pannable_area.add_with_viewport(vbox)
        self.pannable_area.show_all()

    def run(self):
        """Runs dialog."""

        # activate first tab
        # FIXME: read from config
        self.window.show_all()
        self.button_items.set_active(True)

    def save_selected_fields(self, widget, column):
        """Save selected choice."""

        model = self.selector.get_model(0)
        selected_items = [model[index][0] for index in [item[0] for item in \
            self.selector.get_selected_rows(0)]]
        self.config.set_fields(selected_items)
